@@ -21,7 +21,7 @@
 from django import http
 from django.conf import settings
 from django.core.urlresolvers import reverse
-from novaclient import exceptions as novaclient_exceptions
+
 from mox import IsA
 
 from horizon import api
@@ -67,10 +67,10 @@ class SecurityGroupsViewTests(test.TestCase):
     def test_create_security_groups_post_exception(self):
         sec_group = self.security_groups.first()
         self.mox.StubOutWithMock(api, 'security_group_create')
-        exc = novaclient_exceptions.ClientException('ClientException')
         api.security_group_create(IsA(http.HttpRequest),
                                   sec_group.name,
-                                  sec_group.description).AndRaise(exc)
+                                  sec_group.description) \
+                                .AndRaise(self.exceptions.nova)
         self.mox.ReplayAll()
 
         formData = {'method': 'CreateGroup',
@@ -100,21 +100,16 @@ class SecurityGroupsViewTests(test.TestCase):
 
     def test_edit_rules_get_exception(self):
         sec_group = self.security_groups.first()
-        sec_group_list = self.security_groups.list()
 
         self.mox.StubOutWithMock(api, 'security_group_get')
-        exc = novaclient_exceptions.ClientException('ClientException')
-        api.security_group_get(IsA(http.HttpRequest),
-                               sec_group.id).AndRaise(exc)
         self.mox.StubOutWithMock(api, 'security_group_list')
-        api.security_group_list(
-                        IsA(http.HttpRequest)).AndReturn(sec_group_list)
-        api.security_group_list(
-                        IsA(http.HttpRequest)).AndReturn(sec_group_list)
+
+        api.security_group_get(IsA(http.HttpRequest),
+                               sec_group.id).AndRaise(self.exceptions.nova)
         self.mox.ReplayAll()
 
         res = self.client.get(self.edit_url)
-        self.assertRedirects(res, INDEX_URL)
+        self.assertRedirectsNoFollow(res, INDEX_URL)
 
     def test_edit_rules_add_rule_cidr(self):
         sec_group = self.security_groups.first()
@@ -122,6 +117,7 @@ class SecurityGroupsViewTests(test.TestCase):
         rule = self.security_group_rules.first()
 
         self.mox.StubOutWithMock(api, 'security_group_rule_create')
+        self.mox.StubOutWithMock(api, 'security_group_list')
         api.security_group_rule_create(IsA(http.HttpRequest),
                                        sec_group.id,
                                        rule.ip_protocol,
@@ -129,7 +125,6 @@ class SecurityGroupsViewTests(test.TestCase):
                                        int(rule.to_port),
                                        rule.ip_range['cidr'],
                                        None).AndReturn(rule)
-        self.mox.StubOutWithMock(api, 'security_group_list')
         api.security_group_list(
                         IsA(http.HttpRequest)).AndReturn(sec_group_list)
         self.mox.ReplayAll()
@@ -170,6 +165,34 @@ class SecurityGroupsViewTests(test.TestCase):
         msg = 'Either CIDR or Source Group may be specified, but not both.'
         self.assertFormErrors(res, count=1, message=msg)
 
+    def test_edit_rules_add_rule_self_as_source_group(self):
+        sec_group = self.security_groups.first()
+        sec_group_list = self.security_groups.list()
+        rule = self.security_group_rules.get(id=3)
+
+        self.mox.StubOutWithMock(api, 'security_group_rule_create')
+        self.mox.StubOutWithMock(api, 'security_group_list')
+        api.security_group_rule_create(IsA(http.HttpRequest),
+                                       sec_group.id,
+                                       rule.ip_protocol,
+                                       int(rule.from_port),
+                                       int(rule.to_port),
+                                       None,
+                                       u'%s' % sec_group.id).AndReturn(rule)
+        api.security_group_list(
+                        IsA(http.HttpRequest)).AndReturn(sec_group_list)
+        self.mox.ReplayAll()
+
+        formData = {'method': 'AddRule',
+                    'security_group_id': sec_group.id,
+                    'from_port': rule.from_port,
+                    'to_port': rule.to_port,
+                    'ip_protocol': rule.ip_protocol,
+                    'cidr': '0.0.0.0/0',
+                    'source_group': sec_group.id}
+        res = self.client.post(self.edit_url, formData)
+        self.assertRedirectsNoFollow(res, INDEX_URL)
+
     def test_edit_rules_invalid_port_range(self):
         sec_group = self.security_groups.first()
         sec_group_list = self.security_groups.list()
@@ -194,21 +217,88 @@ class SecurityGroupsViewTests(test.TestCase):
         self.assertNoMessages()
         self.assertContains(res, "greater than or equal to")
 
+    @test.create_stubs({api: ('security_group_get', 'security_group_list')})
+    def test_edit_rules_invalid_icmp_rule(self):
+        sec_group = self.security_groups.first()
+        sec_group_list = self.security_groups.list()
+        icmp_rule = self.security_group_rules.list()[1]
+
+        api.security_group_get(IsA(http.HttpRequest),
+                               sec_group.id).AndReturn(sec_group)
+        api.security_group_list(
+                        IsA(http.HttpRequest)).AndReturn(sec_group_list)
+        api.security_group_get(IsA(http.HttpRequest),
+                               sec_group.id).AndReturn(sec_group)
+        api.security_group_list(
+                        IsA(http.HttpRequest)).AndReturn(sec_group_list)
+        api.security_group_get(IsA(http.HttpRequest),
+                               sec_group.id).AndReturn(sec_group)
+        api.security_group_list(
+                        IsA(http.HttpRequest)).AndReturn(sec_group_list)
+        api.security_group_get(IsA(http.HttpRequest),
+                               sec_group.id).AndReturn(sec_group)
+        api.security_group_list(
+                        IsA(http.HttpRequest)).AndReturn(sec_group_list)
+        self.mox.ReplayAll()
+
+        formData = {'method': 'AddRule',
+                    'security_group_id': sec_group.id,
+                    'from_port': 256,
+                    'to_port': icmp_rule.to_port,
+                    'ip_protocol': icmp_rule.ip_protocol,
+                    'cidr': icmp_rule.ip_range['cidr'],
+                    'source_group': ''}
+        res = self.client.post(self.edit_url, formData)
+        self.assertNoMessages()
+        self.assertContains(res, "The ICMP type not in range (-1, 255)")
+
+        formData = {'method': 'AddRule',
+                    'security_group_id': sec_group.id,
+                    'from_port': icmp_rule.from_port,
+                    'to_port': 256,
+                    'ip_protocol': icmp_rule.ip_protocol,
+                    'cidr': icmp_rule.ip_range['cidr'],
+                    'source_group': ''}
+        res = self.client.post(self.edit_url, formData)
+        self.assertNoMessages()
+        self.assertContains(res, "The ICMP code not in range (-1, 255)")
+
+        formData = {'method': 'AddRule',
+                    'security_group_id': sec_group.id,
+                    'from_port': icmp_rule.from_port,
+                    'to_port': None,
+                    'ip_protocol': icmp_rule.ip_protocol,
+                    'cidr': icmp_rule.ip_range['cidr'],
+                    'source_group': ''}
+        res = self.client.post(self.edit_url, formData)
+        self.assertNoMessages()
+        self.assertContains(res, "The ICMP code is invalid")
+
+        formData = {'method': 'AddRule',
+                    'security_group_id': sec_group.id,
+                    'from_port': None,
+                    'to_port': icmp_rule.to_port,
+                    'ip_protocol': icmp_rule.ip_protocol,
+                    'cidr': icmp_rule.ip_range['cidr'],
+                    'source_group': ''}
+        res = self.client.post(self.edit_url, formData)
+        self.assertNoMessages()
+        self.assertContains(res, "The ICMP type is invalid")
+
     def test_edit_rules_add_rule_exception(self):
         sec_group = self.security_groups.first()
         sec_group_list = self.security_groups.list()
         rule = self.security_group_rules.first()
-        exc = novaclient_exceptions.ClientException('ClientException')
 
         self.mox.StubOutWithMock(api, 'security_group_rule_create')
+        self.mox.StubOutWithMock(api, 'security_group_list')
         api.security_group_rule_create(IsA(http.HttpRequest),
                                        sec_group.id,
                                        rule.ip_protocol,
                                        int(rule.from_port),
                                        int(rule.to_port),
                                        rule.ip_range['cidr'],
-                                       None).AndRaise(exc)
-        self.mox.StubOutWithMock(api, 'security_group_list')
+                                       None).AndRaise(self.exceptions.nova)
         api.security_group_list(
                         IsA(http.HttpRequest)).AndReturn(sec_group_list)
         self.mox.ReplayAll()
@@ -241,9 +331,8 @@ class SecurityGroupsViewTests(test.TestCase):
         rule = self.security_group_rules.first()
 
         self.mox.StubOutWithMock(api, 'security_group_rule_delete')
-        exc = novaclient_exceptions.ClientException('ClientException')
         api.security_group_rule_delete(IsA(http.HttpRequest),
-                                       rule.id).AndRaise(exc)
+                                       rule.id).AndRaise(self.exceptions.nova)
         self.mox.ReplayAll()
 
         form_data = {"action": "rules__delete__%s" % rule.id}
@@ -271,9 +360,8 @@ class SecurityGroupsViewTests(test.TestCase):
         sec_group = self.security_groups.get(name="other_group")
 
         self.mox.StubOutWithMock(api, 'security_group_delete')
-        exc = novaclient_exceptions.ClientException('ClientException')
         api.security_group_delete(IsA(http.HttpRequest),
-                                  sec_group.id).AndRaise(exc)
+                                  sec_group.id).AndRaise(self.exceptions.nova)
 
         self.mox.ReplayAll()
 
