@@ -18,50 +18,84 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import datetime
+from functools import wraps
+import os
+
 
-import cloudfiles as swift_client
 from django import http
 from django import test as django_test
 from django.conf import settings
 from django.contrib.messages.storage import default_storage
+from django.contrib.auth.middleware import AuthenticationMiddleware
 from django.core.handlers import wsgi
 from django.test.client import RequestFactory
-from glance import client as glance_client
+from django.utils import unittest
+
+import glanceclient
 from keystoneclient.v2_0 import client as keystone_client
 from novaclient.v1_1 import client as nova_client
+from quantumclient.v2_0 import client as quantum_client
+from swiftclient import client as swift_client
+from cinderclient import client as cinder_client
+
+from selenium.webdriver.firefox.webdriver import WebDriver
+
 import httplib2
 import mox
 
+from openstack_auth import utils, user
+
 from horizon import api
 from horizon import context_processors
 from horizon import middleware
-from horizon import users
 from horizon.tests.test_data.utils import load_test_data
 
-from .time import time
-from .time import today
-from .time import utcnow
-
 
 # Makes output of failing mox tests much easier to read.
 wsgi.WSGIRequest.__repr__ = lambda self: "<class 'django.http.HttpRequest'>"
 
 
+def create_stubs(stubs_to_create={}):
+    if not isinstance(stubs_to_create, dict):
+        raise TypeError("create_stub must be passed a dict, but a %s was "
+                        "given." % type(stubs_to_create).__name__)
+
+    def inner_stub_out(fn):
+        @wraps(fn)
+        def instance_stub_out(self):
+            for key in stubs_to_create:
+                if not (isinstance(stubs_to_create[key], tuple) or
+                        isinstance(stubs_to_create[key], list)):
+                    raise TypeError("The values of the create_stub "
+                                    "dict must be lists or tuples, but "
+                                    "is a %s."
+                                    % type(stubs_to_create[key]).__name__)
+
+                for value in stubs_to_create[key]:
+                    self.mox.StubOutWithMock(key, value)
+            return fn(self)
+        return instance_stub_out
+    return inner_stub_out
+
+
 class RequestFactoryWithMessages(RequestFactory):
     def get(self, *args, **kwargs):
         req = super(RequestFactoryWithMessages, self).get(*args, **kwargs)
+        req.user = utils.get_user(req)
         req.session = []
         req._messages = default_storage(req)
         return req
 
     def post(self, *args, **kwargs):
         req = super(RequestFactoryWithMessages, self).post(*args, **kwargs)
+        req.user = utils.get_user(req)
         req.session = []
         req._messages = default_storage(req)
         return req
 
 
+@unittest.skipIf(os.environ.get('SKIP_UNITTESTS', False),
+                     "The SKIP_UNITTESTS env variable is set.")
 class TestCase(django_test.TestCase):
     """
     Specialized base test case class for Horizon which gives access to
@@ -96,10 +130,10 @@ class TestCase(django_test.TestCase):
         self._real_horizon_context_processor = context_processors.horizon
         context_processors.horizon = lambda request: self.context
 
-        self._real_get_user_from_request = users.get_user_from_request
+        self._real_get_user = utils.get_user
         tenants = self.context['authorized_tenants']
         self.setActiveUser(id=self.user.id,
-                           token=self.token.id,
+                           token=self.token,
                            username=self.user.name,
                            tenant_id=self.tenant.id,
                            service_catalog=self.service_catalog,
@@ -108,41 +142,31 @@ class TestCase(django_test.TestCase):
         self.request.session = self.client._session()
         self.request.session['token'] = self.token.id
         middleware.HorizonMiddleware().process_request(self.request)
+        AuthenticationMiddleware().process_request(self.request)
+        os.environ["HORIZON_TEST_RUN"] = "True"
 
     def tearDown(self):
         self.mox.UnsetStubs()
         httplib2.Http._conn_request = self._real_conn_request
         context_processors.horizon = self._real_horizon_context_processor
-        users.get_user_from_request = self._real_get_user_from_request
+        utils.get_user = self._real_get_user
         self.mox.VerifyAll()
+        del os.environ["HORIZON_TEST_RUN"]
 
     def setActiveUser(self, id=None, token=None, username=None, tenant_id=None,
                         service_catalog=None, tenant_name=None, roles=None,
-                        authorized_tenants=None):
-        users.get_user_from_request = lambda x: \
-                users.User(id=id,
-                           token=token,
-                           user=username,
-                           tenant_id=tenant_id,
-                           service_catalog=service_catalog,
-                           roles=roles,
-                           authorized_tenants=authorized_tenants,
-                           request=self.request)
-
-    def override_times(self):
-        """ Overrides the "current" time with immutable values. """
-        now = datetime.datetime.utcnow()
-        time.override_time = \
-                datetime.time(now.hour, now.minute, now.second)
-        today.override_time = datetime.date(now.year, now.month, now.day)
-        utcnow.override_time = now
-        return now
-
-    def reset_times(self):
-        """ Undoes the changes made by ``override_times``. """
-        time.override_time = None
-        today.override_time = None
-        utcnow.override_time = None
+                        authorized_tenants=None, enabled=True):
+        def get_user(request):
+            return user.User(id=id,
+                             token=token,
+                             user=username,
+                             tenant_id=tenant_id,
+                             service_catalog=service_catalog,
+                             roles=roles,
+                             enabled=enabled,
+                             authorized_tenants=authorized_tenants,
+                             endpoint=settings.OPENSTACK_KEYSTONE_URL)
+        utils.get_user = get_user
 
     def assertRedirectsNoFollow(self, response, expected_url):
         """
@@ -180,7 +204,7 @@ class TestCase(django_test.TestCase):
                 message_cookie = self.client.cookies['messages'].value
                 messages = storage._decode(message_cookie)
         # Check for messages in the context
-        elif hasattr(response, "context") and  "messages" in response.context:
+        elif hasattr(response, "context") and "messages" in response.context:
             messages = response.context["messages"]
         # Check for messages attached to the request on a TemplateResponse
         elif hasattr(response, "_request") and hasattr(response._request,
@@ -257,9 +281,7 @@ class APITestCase(TestCase):
     def setUp(self):
         super(APITestCase, self).setUp()
 
-        def fake_keystoneclient(request, username=None, password=None,
-                                tenant_id=None, token_id=None, endpoint=None,
-                                admin=False):
+        def fake_keystoneclient(request, admin=False):
             """
             Wrapper function which returns the stub keystoneclient. Only
             necessary because the function takes too many arguments to
@@ -271,17 +293,23 @@ class APITestCase(TestCase):
         self._original_glanceclient = api.glance.glanceclient
         self._original_keystoneclient = api.keystone.keystoneclient
         self._original_novaclient = api.nova.novaclient
+        self._original_quantumclient = api.quantum.quantumclient
+        self._original_cinderclient = api.nova.cinderclient
 
         # Replace the clients with our stubs.
         api.glance.glanceclient = lambda request: self.stub_glanceclient()
         api.keystone.keystoneclient = fake_keystoneclient
         api.nova.novaclient = lambda request: self.stub_novaclient()
+        api.quantum.quantumclient = lambda request: self.stub_quantumclient()
+        api.nova.cinderclient = lambda request: self.stub_cinderclient()
 
     def tearDown(self):
         super(APITestCase, self).tearDown()
         api.glance.glanceclient = self._original_glanceclient
         api.nova.novaclient = self._original_novaclient
         api.keystone.keystoneclient = self._original_keystoneclient
+        api.quantum.quantumclient = self._original_quantumclient
+        api.nova.cinderclient = self._original_cinderclient
 
     def stub_novaclient(self):
         if not hasattr(self, "novaclient"):
@@ -289,6 +317,12 @@ class APITestCase(TestCase):
             self.novaclient = self.mox.CreateMock(nova_client.Client)
         return self.novaclient
 
+    def stub_cinderclient(self):
+        if not hasattr(self, "cinderclient"):
+            self.mox.StubOutWithMock(cinder_client, 'Client')
+            self.cinderclient = self.mox.CreateMock(cinder_client.Client)
+        return self.cinderclient
+
     def stub_keystoneclient(self):
         if not hasattr(self, "keystoneclient"):
             self.mox.StubOutWithMock(keystone_client, 'Client')
@@ -297,17 +331,43 @@ class APITestCase(TestCase):
 
     def stub_glanceclient(self):
         if not hasattr(self, "glanceclient"):
-            self.mox.StubOutWithMock(glance_client, 'Client')
-            self.glanceclient = self.mox.CreateMock(glance_client.Client)
-            self.glanceclient.token = self.tokens.first().id
+            self.mox.StubOutWithMock(glanceclient, 'Client')
+            self.glanceclient = self.mox.CreateMock(glanceclient.Client)
         return self.glanceclient
 
+    def stub_quantumclient(self):
+        if not hasattr(self, "quantumclient"):
+            self.mox.StubOutWithMock(quantum_client, 'Client')
+            self.quantumclient = self.mox.CreateMock(quantum_client.Client)
+        return self.quantumclient
+
     def stub_swiftclient(self, expected_calls=1):
         if not hasattr(self, "swiftclient"):
             self.mox.StubOutWithMock(swift_client, 'Connection')
             self.swiftclient = self.mox.CreateMock(swift_client.Connection)
             while expected_calls:
-                swift_client.Connection(auth=mox.IgnoreArg())\
+                swift_client.Connection(None,
+                                        mox.IgnoreArg(),
+                                        None,
+                                        preauthtoken=mox.IgnoreArg(),
+                                        preauthurl=mox.IgnoreArg(),
+                                        auth_version="2.0") \
                             .AndReturn(self.swiftclient)
                 expected_calls -= 1
         return self.swiftclient
+
+
+@unittest.skipUnless(os.environ.get('WITH_SELENIUM', False),
+                     "The WITH_SELENIUM env variable is not set.")
+class SeleniumTestCase(django_test.LiveServerTestCase):
+    @classmethod
+    def setUpClass(cls):
+        if os.environ.get('WITH_SELENIUM', False):
+            cls.selenium = WebDriver()
+        super(SeleniumTestCase, cls).setUpClass()
+
+    @classmethod
+    def tearDownClass(cls):
+        super(SeleniumTestCase, cls).tearDownClass()
+        if os.environ.get('WITH_SELENIUM', False):
+            cls.selenium.quit()
