@@ -42,3 +42,8 @@ urlpatterns += staticfiles_urlpatterns()
 # development. Only active if DEBUG==True and the URL prefix is a local
 # path. Production media should NOT be served by Django.
 urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
+
+if settings.DEBUG:
+    urlpatterns += patterns('',
+        url(r'^500/$', 'django.views.defaults.server_error')
+    )
